import datetime
import os
import pickle
import sys


def save_obj(obj, saved_name):
    """save_obj.

    this function is used to save any python object to your hard desk

    Parameters
    ----------
    obj: [Any]
        any python object
    saved_name: [str]
        name of the object

    Returns
    -------
    the object will be saved to the given path/current working directory
    with the given name

    Examples
    --------
    >>> path = "path/to/your/disk"
    >>> data={"key1":[1,2,3,5],"key2":[6,2,9,7]}
    >>> save_obj(data, f'{path}/flow_acc_table')
    """
    with open(saved_name + ".pkl", "wb") as f:
        pickle.dump(obj, f, pickle.HIGHEST_PROTOCOL)


def load_obj(saved_name):
    """load_obj.

        this function is used to save any python object to your hard desk

    Parameters
    ----------
    1-saved_name:
        ['String'] name of the object

    Returns
    -------
    the object will be loaded

    Examples
    --------
    >>> path = r"c:\my_computer\files"
    >>> load_obj(f'{path}/flow_acc_table')
    """
    with open(saved_name + ".pkl", "rb") as f:
        return pickle.load(f)


def dateformated(x):
    """dateformated.

    this function converts the the date read from a list to a datetime format

    Parameters
    ----------
    x: [list]
        is a list of tuples of string date read from database

    Returns
    -------
        list od dates as a datetime format  YYYY-MM-DD HH:MM:SS
    """
    x = [i[0] for i in x]
    #
    x1 = []
    for i in x:
        if len(i) == 19:
            x1.append(
                datetime.datetime(
                    int(i[:4]),
                    int(i[5:7]),
                    int(i[8:10]),
                    int(i[11:13]),
                    int(i[14:16]),
                    int(i[17:18]),
                )
            )
    #        elif len(i)==13:
    #            x1.append(datetime.datetime(int(i[:4]),int(i[5:7]),int(i[8:10]),int(i[11:13]),int(0),int(0) ))
    #        else:
    #            x1.append(datetime.datetime(int(i[:4]),int(i[5:7]),int(i[8:10]),int(0),int(0),int(0) ))
    #    del i,x
    return x1


def print_progress_bar(
    i: int,
    total: int,
    prefix: str = "",
    suffix: str = "",
    decimals: int = 1,
    length: int = 100,
    fill: str = "█",
):
    """print_progress_bar.

    Parameters
    ----------
    i: [int]
        Iteration number
    total: [int]
        Total iterations
    prefix: [str]
        Name after bar
    suffix: [str]
        Decimals of percentage
    decimals: [int]

    length: [int]
        width of the waitbar
    fill: [str]
        bar fill
    """
    # Adjust when it is a linux computer
    if os.name == "posix" and total == 0:
        total = 0.0001

    percent = ("{0:." + str(decimals) + "f}").format(100 * (i / float(total)))
    filled = int(length * i // total)
    bar = fill * filled + "-" * (length - filled)

    sys.stdout.write("\r%s |%s| %s%% %s" % (prefix, bar, percent, suffix))
    sys.stdout.flush()

    if i == total:
        print()


def class_method_parse(initial_args):
    """check and assign values of parameters to the self object.

        check values of a method and assign the valuse of the parameters to the self object (the method has self/cls
        as first parameter)

    Parameters
    ----------
    initial_args: [Dict]
        dictionary contains all the parameters of the function, positional and key word parameters, each parameter is a
        key(i.e 'name' in the below example), and the value is a dict that has at least a key called "type",
        and a value that is an available data type in python, (i.e 'name' : {"type": str}),
        - If the parameter has a default value, the dict has to have another key: value i.e "default": <any value>
        - if there is no "default" key in the parameter dict, the default value will be taken None
        >>> initial_args = {
        >>>     'name' : {"type": str},
        >>>     'version' : {"default": 3, "type": int}
        >>> }

    Returns
    -------
    assign the valuse of the parameters to the self object
    """

    def apply_func(func):
        """apply the function that the decorator preceded.

        Parameters
        ----------
        func: [function]
            the function that the decorator precedes

        Returns
        -------
        returns the same outputs of the input function
        """

        def wrapper(*args, **kwargs):

            self = args[0]
            # get wrong kwargs
            wrong_kwargs = set(kwargs) - set(initial_args)
            if len(wrong_kwargs) > 0:
                print(initial_args)
                raise KeyError(f"Invalid parameter {wrong_kwargs}")

            for key, val in initial_args.items():
                # if the parameter is given by user
                if key in kwargs.keys():
                    default = initial_args.get(key)
                    # check the type
                    key_type = default.get("type")
                    # make the type as a list
                    if not isinstance(key_type, list):
                        key_type = [key_type]
                    # get the given value
                    val = kwargs.get(key)
                    if type(val) in key_type:
                        # set the given value
                        setattr(self, key, val)
                    else:
                        raise TypeError(
                            f"The parameter {key} should be of type {key_type}"
                        )
                else:
                    # positional args
                    if "default" in val.keys():
                        setattr(self, key, val.get("default"))

            res = func(*args, **kwargs)
            return res

        return wrapper

    return apply_func


def class_attr_initialize(attributes):
    """check and assign values of parameters to the self object.

        check values of a method and assign the valuse of the parameters to the self object
        (the method has self/cls
        as first parameter)

    Parameters
    ----------
    initial_args: [Dict]
        dictionary contains all the parameters of the function, positional and key word
        parameters, each parameter is a key(i.e 'name' in the below example), and the
        value is a dict that has at least a key called "type", and a value that is an
        available data type in python, (i.e 'name' : {"type": str}),
        - If the parameter has a default value, the dict has to have another
        key: value i.e "default": <any value>
        >>> initial_args = {
        >>>     'name' : {"type": str},
        >>>     'version' : {"default": 3, "type": int}
        >>> }

    Returns
    -------
    assign the valuse of the parameters to the self object
    """

    def apply_func(func):
        """apply the function that the decorator preceded.

        Parameters
        ----------
        func: [function]
            the function that the decorator precedes

        Returns
        -------
        returns the same outputs of the input function
        """

        def wrapper(*args, **kwargs):
            self = args[0]
            # initialize attributes
            for key, val in attributes.items():
                setattr(self, key, val)

            func(*args, **kwargs)

        return wrapper

    return apply_func



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['serapeum_utils']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'serapeum-utils',
    'version': '0.1.1',
    'description': 'utility package',
    'long_description': '![GitHub release (latest by date)](https://img.shields.io/github/v/release/Serapieum-of-alex/serapeum_utils)\n[![Python Versions](https://img.shields.io/pypi/pyversions/HAPI-Nile.png)](https://img.shields.io/pypi/pyversions/HAPI-Nile)\n[![License: GPL v3](https://img.shields.io/badge/License-GPLv3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)\n[![pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)](https://github.com/pre-commit/pre-commit)\n[![Language grade: Python](https://img.shields.io/lgtm/grade/python/g/Serapieum-of-alex/Hapi.svg?logo=lgtm&logoWidth=18)](https://lgtm.com/projects/g/Serapieum-of-alex/Hapi/context:python)\n\nCurrent build status\n====================\n\n\n<table><tr><td>All platforms:</td>\n    <td>\n      <a href="https://dev.azure.com/conda-forge/feedstock-builds/_build/latest?definitionId=12419&branchName=master">\n        <img src="https://dev.azure.com/conda-forge/feedstock-builds/_apis/build/status/hapi-feedstock?branchName=master">\n      </a>\n    </td>\n  </tr>\n</table>\n\n![GitHub last commit](https://img.shields.io/github/last-commit/Serapieum-of-alex/serapeum_utils)\n![GitHub forks](https://img.shields.io/github/forks/Serapieum-of-alex/serapeum_utils?style=social)\n![GitHub Repo stars](https://img.shields.io/github/stars/Serapieum-of-alex/serapeum_utils?style=social)\n\n\n\n[![Github all releases](https://img.shields.io/github/downloads/Naereen/StrapDown.js/total.svg)](https://GitHub.com/Naereen/StrapDown.js/releases/)\n\n\nCurrent release info\n====================\n\n| Name | Downloads                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                | Version | Platforms |\n| --- |------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------| --- | --- |\n| [![Conda Recipe](https://img.shields.io/badge/recipe-serapeum_utils-green.svg)](https://anaconda.org/conda-forge/serapeum_utils) | [![Conda Downloads](https://img.shields.io/conda/dn/conda-forge/serapeum_utils.svg)](https://anaconda.org/conda-forge/serapeum_utils) [![Downloads](https://pepy.tech/badge/serapeum_utils-nile)](https://pepy.tech/project/serapeum_utils-nile) [![Downloads](https://pepy.tech/badge/serapeum_utils-nile/month)](https://pepy.tech/project/serapeum_utils-nile)  [![Downloads](https://pepy.tech/badge/serapeum_utils-nile/week)](https://pepy.tech/project/serapeum_utils-nile)  ![PyPI - Downloads](https://img.shields.io/pypi/dd/serapeum_utils-nile?color=blue&style=flat-square) ![GitHub all releases](https://img.shields.io/github/downloads/Serapieum-of-alex/serapeum_utils/total) ![GitHub release (latest by date)](https://img.shields.io/github/downloads/Serapieum-of-alex/serapeum_utils/1.2.3/total) | [![Conda Version](https://img.shields.io/conda/vn/conda-forge/serapeum_utils.svg)](https://anaconda.org/conda-forge/serapeum_utils) [![PyPI version](https://badge.fury.io/py/serapeum_utils-Nile.svg)](https://badge.fury.io/py/serapeum_utils-Nile) [![Anaconda-Server Badge](https://anaconda.org/conda-forge/serapeum_utils/badges/version.svg)](https://anaconda.org/conda-forge/serapeum_utils) | [![Conda Platforms](https://img.shields.io/conda/pn/conda-forge/serapeum_utils.svg)](https://anaconda.org/conda-forge/serapeum_utils) [![Join the chat at https://gitter.im/serapeum_utils-Nile/serapeum_utils](https://badges.gitter.im/serapeum_utils-Nile/serapeum_utils.svg)](https://gitter.im/serapeum_utils-Nile/serapeum_utils?utm_source=badge&utm_medium=badge&utm_campaign=pr-badge&utm_content=badge) |\n\n\n\n\n\nserapeum_utils - utility library for Python\n=====================================================================\n**serapeum_utils** is an open-source Python utility package\n\n\nserapeum_utils\n\nMain Features\n-------------\n  -\n\n\nInstalling serapeum_utils\n===============\n\nInstalling `serapeum_utils` from the `conda-forge` channel can be achieved by:\n\n```\nconda install -c conda-forge serapeum_utils\n```\n\nIt is possible to list all of the versions of `serapeum_utils` available on your platform with:\n\n```\nconda search serapeum_utils --channel conda-forge\n```\n\n## Install from Github\nto install the last development to time you can install the library from github\n```\npip install git+https://github.com/Serapieum-of-alex/serapeum_utils\n```\n\n## pip\nto install the last release you can easly use pip\n```\npip install serapeum_utils==0.1.1\n```\n\nQuick start\n===========\n\n```\n  >>> import serapeum_utils\n```\n',
    'author': 'Mostafa Farrag',
    'author_email': 'moah.farag@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/Serapieum-of-alex/serapeum_utils',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
